@@ -1,7 +1,7 @@
 # - coding: utf-8 -
 
-# Copyright (C) 2007, 2008 Toms Bauģis <toms.baugis at gmail.com>
-# Copyright (C) 2007, 2008 Patryk Zawadzki <patrys at pld-linux.org>
+# Copyright (C) 2007-2009 Toms Bauģis <toms.baugis at gmail.com>
+# Copyright (C) 2007-2009 Patryk Zawadzki <patrys at pld-linux.org>
 # Copyright (C) 2008 Pēteris Caune <cuu508 at gmail.com>
 
 # This file is part of Project Hamster.
@@ -38,18 +38,20 @@ from hamster import dispatcher, storage, SHARED_DATA_DIR
 import hamster.eds
 from hamster.Configuration import GconfStore
 
-from hamster.stuff import *
+from hamster import stuff
 from hamster.KeyBinder import *
 from hamster.hamsterdbus import HAMSTER_URI, HamsterDbusController 
 
 # controllers for other windows
-from hamster.add_custom_fact import CustomFactController
+from hamster.edit_activity import CustomFactController
 from hamster.stats import StatsViewer
 from hamster.about import show_about
 from hamster.preferences import PreferencesEditor
 
 import idle
 
+import pango
+
 try:
     import pynotify
     PYNOTIFY = True
@@ -75,6 +77,7 @@ class Notifier(object):
             self._notify.add_action("edit", _("Edit"), edit_cb)
             #translators: this is switch activity action in the notifier bubble
             self._notify.add_action("switch", _("Switch"), switch_cb)
+            
         self._notify.show()
 
 
@@ -111,7 +114,7 @@ class PanelButton(gtk.ToggleButton):
             else:
                 label = "%s %s" % (self.activity, self.duration)
         
-        label = escape_pango(label)
+        label = stuff.escape_pango(label)
         label = '<span gravity=\"south\">' + label + '</span>'
         self.label.set_markup(label)
 
@@ -129,7 +132,6 @@ class PanelButton(gtk.ToggleButton):
                           popup_dir in [gnomeapplet.ORIENT_RIGHT]
 
 
-        import pango
         context = self.label.get_pango_context()
         metrics = context.get_metrics(self.label.style.font_desc,
                                       pango.Context.get_language(context))
@@ -184,19 +186,6 @@ class PanelButton(gtk.ToggleButton):
 
 
 class HamsterApplet(object):
-    def name_painter(self, column, cell, model, iter):
-        activity_name = model.get_value(iter, 1)
-        description = model.get_value(iter, 5)
-
-        text = """%s""" % activity_name
-        if description:
-            text+= """\n<span style="italic" size="small">%s</span>""" % (description)
-            
-        cell.set_property('markup', text)
-            
-        return
-
-
     def __init__(self, applet):
         self.config = GconfStore.get_instance()
         
@@ -207,36 +196,18 @@ class HamsterApplet(object):
 
         self.preferences_editor = None
         self.applet.about = None
+        self.open_fact_editors = []
 
         self.button = PanelButton()
         
         # load window of activity switcher and todays view
-        self.glade = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, "menu.glade"))
-        self.window = self.glade.get_widget('hamster-window')
-        self.window.set_keep_above(True)
+        self._gui = stuff.load_ui_file("applet.ui")
+        self.window = self._gui.get_object('hamster-window')
 
         self.set_dropdown()
 
         # init today's tree
-        self.treeview = self.glade.get_widget('today')
-        self.treeview.set_tooltip_column(1)
-        
-        self.treeview.append_column(gtk.TreeViewColumn("Time", gtk.CellRendererText(), text=2))
-
-        nameColumn = gtk.TreeViewColumn(_("Name"))
-        nameColumn.set_expand(True)
-        nameCell = gtk.CellRendererText()
-        nameColumn.pack_start(nameCell, True)
-        nameCell.set_property("ellipsize", pango.ELLIPSIZE_END)
-        nameColumn.set_cell_data_func(nameCell, self.name_painter)
-        self.treeview.append_column(nameColumn)
-
-        
-        edit_cell = gtk.CellRendererPixbuf()
-        edit_cell.set_property("stock_id", "gtk-edit")
-        edit_cell.set_property("mode", gtk.CELL_RENDERER_MODE_ACTIVATABLE)
-        self.edit_column = gtk.TreeViewColumn("", edit_cell)
-        self.treeview.append_column(self.edit_column)
+        self.setup_activity_tree()
 
         # DBus Setup
         try:
@@ -244,6 +215,9 @@ class HamsterApplet(object):
             name = dbus.service.BusName(HAMSTER_URI, dbus.SessionBus())
             self.dbusController = HamsterDbusController(bus_name = name)
 
+            # Set up connection to the screensaver
+            self.dbusIdleListener = idle.DbusIdleListener()
+
             # let's also attach our listeners here
             bus = dbus.SessionBus()
             bus.add_signal_receiver(self.on_idle_changed,
@@ -254,7 +228,6 @@ class HamsterApplet(object):
     
         # Load today's data, activities and set label
         self.last_activity = None
-        self.today = datetime.date.today()
 
         self.load_day()
         self.update_label()
@@ -298,7 +271,7 @@ class HamsterApplet(object):
         self.button.connect('toggled', self.on_toggle)
 
         self.button.connect('button_press_event', self.on_button_press)
-        self.glade.signal_autoconnect(self)
+        self._gui.connect_signals(self)
 
         # init hotkey
         dispatcher.add_handler('keybinding_activated', self.on_keybinding_activated)
@@ -307,12 +280,58 @@ class HamsterApplet(object):
         dispatcher.add_handler('gconf_timeout_enabled_changed', self.on_timeout_enabled_changed)
         self.on_timeout_enabled_changed(None, self.config.get_timeout_enabled())
         
+        
         # init nagging timeout
         if PYNOTIFY:
             self.notify = Notifier('HamsterApplet', gtk.STOCK_DIALOG_QUESTION, self.button)
             dispatcher.add_handler('gconf_notify_interval_changed', self.on_notify_interval_changed)
             self.on_notify_interval_changed(None, self.config.get_notify_interval())
 
+    def give_more_info(self):
+        def on_response(self, widget):
+            self.destroy()
+        message_dialog = gtk.MessageDialog(buttons = gtk.BUTTONS_OK)
+        message_dialog.set_property("title", _("What to type in the activity box?"))
+        message_dialog.connect("response", on_response)
+        
+        more_info = _("""There is simple syntax that enables you to add details to your activities:
+        
+"@" symbol marks category. Example: "watering flowers@home" will start tracking activity "watering flowers" in category "home".
+
+Comma (",") marks beginning of description. Example: "watering flowers, begonias and forgetmenots" will start tracking activity "watering flowers" and add description "begonias and forgetmenots" to it.
+
+Both can be combined: "watering flowers@home, begonias and forgetmenots" will work just fine!
+
+Now, start tracking!
+        """)
+        
+        message_dialog.set_markup(more_info)
+        message_dialog.show()
+        
+    def on_more_info_button_clicked(self, button):
+        self.give_more_info()
+
+    def setup_activity_tree(self):
+        self.treeview = self._gui.get_object('today')
+        self.treeview.set_tooltip_column(1)
+        
+        self.treeview.append_column(gtk.TreeViewColumn("Time", gtk.CellRendererText(), text=2))
+
+        nameColumn = stuff.ActivityColumn(name=1, description=5, category=6)
+        self.treeview.append_column(nameColumn)
+
+        
+        duration_cell = gtk.CellRendererText()
+        duration_cell.set_property("xalign", 1)
+        timeColumn = gtk.TreeViewColumn(_("Duration"), duration_cell, text=3)
+        
+        self.treeview.append_column(timeColumn)
+
+        edit_cell = gtk.CellRendererPixbuf()
+        edit_cell.set_property("stock_id", "gtk-edit")
+        self.edit_column = gtk.TreeViewColumn("", edit_cell)
+        self.treeview.append_column(self.edit_column)
+
 
     def on_idle_changed(self, state):
         print "Idle state changed. Idle: ", state
@@ -323,7 +342,10 @@ class HamsterApplet(object):
         
     def set_dropdown(self):
         # set up drop down menu
-        self.activity_list = self.glade.get_widget('activity-list')
+        self.activity_list = self._gui.get_object('activity-list')
+        self.activity_list.child.connect('activate', self.on_activity_entered)
+        self.activity_list.child.connect('key-press-event', self.on_activity_list_key_pressed)
+
         self.activity_list.set_model(gtk.ListStore(gobject.TYPE_STRING,
                                                    gobject.TYPE_STRING,
                                                    gobject.TYPE_STRING))
@@ -332,7 +354,8 @@ class HamsterApplet(object):
         activity_cell = gtk.CellRendererText()
         self.activity_list.pack_start(activity_cell, True)
         self.activity_list.add_attribute(activity_cell, 'text', 0)
-        category_cell = CategoryCell()  
+
+        category_cell = stuff.CategoryCell()  
         self.activity_list.pack_start(category_cell, False)
         self.activity_list.add_attribute(category_cell, 'text', 1)
         
@@ -346,17 +369,24 @@ class HamsterApplet(object):
         completion = gtk.EntryCompletion()
         completion.set_model(self.activities)
 
-        completion.set_text_column(2)
-        completion.clear()
-
         activity_cell = gtk.CellRendererText()
         completion.pack_start(activity_cell, True)
         completion.add_attribute(activity_cell, 'text', 0)
         completion.set_property("text-column", 2)
 
-        category_cell = CategoryCell()  
+        category_cell = stuff.CategoryCell()  
         completion.pack_start(category_cell, False)
         completion.add_attribute(category_cell, 'text', 1)
+
+
+        def match_func(completion, key, iter):
+            model = completion.get_model()
+            text = model.get_value(iter, 2)
+            if text and text.startswith(key):
+                return True
+            return False
+
+        completion.set_match_func(match_func)
         completion.set_minimum_key_length(1)
         completion.set_inline_completion(True)
 
@@ -372,7 +402,7 @@ class HamsterApplet(object):
             (model, iter) = selection.get_selected()
             fact_id = model[iter][0]
                 
-            custom_fact = CustomFactController(None, fact_id)
+            custom_fact = CustomFactController(self, None, fact_id)
             custom_fact.show()
             return True
         
@@ -381,14 +411,16 @@ class HamsterApplet(object):
     """UI functions"""
     def refresh_hamster(self):
         """refresh hamster every x secs - load today, check last activity etc."""        
-        prev_date = self.today
-        self.today = datetime.date.today()
-
         # stop tracking task if computer is idle for X minutes
         if self.timeout_enabled and self.last_activity and \
            self.last_activity['end_time'] == None:
-            idle_minutes = idle.getIdleSec() / 60.0
-            if idle_minutes > 0:
+            if self.dbusIdleListener.is_idle:
+                # Only subtract idle time from the running task when
+                # idleness is due to time out, not a screen lock.
+                if self.dbusIdleListener.is_screen_locked:
+                    idle_minutes = 0
+                else:
+                    idle_minutes = idle.getIdleSec() / 60.0
                 current_time = datetime.datetime.now()
                 idle_from = current_time - datetime.timedelta(minutes = idle_minutes)
                 storage.touch_fact(self.last_activity, end_time = idle_from)
@@ -401,15 +433,7 @@ class HamsterApplet(object):
                 self.check_user()
 
             # if we have date change - let's finish previous task and start a new one
-            if prev_date and prev_date != self.today: 
-                storage.touch_fact(self.last_activity)
-                activity_name = self.last_activity['name']
-                if self.last_activity['category']:
-                    activity_name += "@%s" % self.last_activity['category']
-                if self.last_activity['description']:
-                    activity_name += ",%s" % self.last_activity['description']
-                storage.add_fact(activity_name)
-            elif self.button.get_active(): # otherwise if we the day view is visible - update day's durations
+            if self.button.get_active(): # otherwise if we the day view is visible - update day's durations
                 self.load_day()
                     
         self.update_label()
@@ -419,14 +443,16 @@ class HamsterApplet(object):
         if self.last_activity and self.last_activity['end_time'] == None:
             delta = datetime.datetime.now() - self.last_activity['start_time']
             duration = delta.seconds /  60
-            label = "%s %s" % (self.last_activity['name'], format_duration(duration))
-            self.button.set_text(self.last_activity['name'], format_duration(duration))
+            label = "%s %s" % (self.last_activity['name'],
+                                                stuff.format_duration(duration, False))
+            self.button.set_text(self.last_activity['name'],
+                                                stuff.format_duration(duration, False))
             
-            self.glade.get_widget('stop_tracking').set_sensitive(1);
+            self._gui.get_object('stop_tracking').set_sensitive(1);
         else:
             label = "%s" % _(u"No activity")
             self.button.set_text(label, None)
-            self.glade.get_widget('stop_tracking').set_sensitive(0);
+            self._gui.get_object('stop_tracking').set_sensitive(0);
         
         
         # Hamster DBusController current activity updating
@@ -442,7 +468,7 @@ class HamsterApplet(object):
             self.notify.msg(msg, self.edit_cb, self.switch_cb)
 
     def edit_cb(self, n, action):
-        custom_fact = CustomFactController(None, self.last_activity['id'])
+        custom_fact = CustomFactController(self, None, self.last_activity['id'])
         custom_fact.show()
 
     def switch_cb(self, n, action):
@@ -452,32 +478,71 @@ class HamsterApplet(object):
     def load_day(self):
         """sets up today's tree and fills it with records
            returns information about last activity"""
-        day = DayStore(self.today);
-        self.treeview.set_model(day.fact_store)
+        today = datetime.date.today()
 
-        if len(day.facts) == 0:
-            self.last_activity = None
-            self.glade.get_widget("todays_scroll").hide()
+        self.last_activity = None
+        last_activity = storage.get_last_activity()
+        if last_activity and last_activity["start_time"].date() >= \
+                                            today - datetime.timedelta(days=1):
+            self.last_activity = last_activity
+
+
+        # ID, Time, Name, Duration, Date, Description, Category
+        fact_store = gtk.ListStore(int, str, str, str, str, str, str)
+        facts = storage.get_facts(today)
+        
+        totals = {}
+        
+        for fact in facts:
+            duration = None
             
-            self.glade.get_widget("fact_totals").set_text(_("No records today"))
+            if fact["delta"]:
+                duration = 24 * fact["delta"].days + fact["delta"].seconds / 60
+            
+            fact_category = fact['category']
+            
+            if fact_category not in totals:
+                totals[fact_category] = 0
+
+            if duration:
+                totals[fact_category] += duration
+
+            current_duration = stuff.format_duration(duration)
+            
+            fact_store.append([fact['id'],
+                                    stuff.escape_pango(fact['name']), 
+                                    fact["start_time"].strftime("%H:%M"), 
+                                    "%s" % current_duration,
+                                    fact["start_time"].strftime("%H:%M"),
+                                    stuff.escape_pango(fact["description"]),
+                                    stuff.escape_pango(fact["category"])])
+
+
+
+        self.treeview.set_model(fact_store)
+
+        
+        if len(facts) == 0:
+            self._gui.get_object("todays_scroll").hide()
+            self._gui.get_object("fact_totals").set_text(_("No records today"))
         else:
-            self.last_activity = day.facts[len(day.facts) - 1]
-            self.glade.get_widget("todays_scroll").show()
+            self._gui.get_object("todays_scroll").show()
             
-            total_string = ""
-            for total in day.totals:
-                total_string += _("%(category)s: %(duration)s, ") % ({'category': total,
-                                                                      'duration': format_duration(day.totals[total])})
+            total_strings = []
+            for total in totals:
+                # listing of today's categories and time spent in them
+                total_strings.append(_("%(category)s: %(duration)s") % \
+                        ({'category': total,
+                          'duration': _("%.1fh") % (totals[total] / 60.0)}))
 
-            total_string = total_string.rstrip(", ") # trailing slash
-            self.glade.get_widget("fact_totals").set_text(total_string)
+            total_string = ", ".join(total_strings)
+            self._gui.get_object("fact_totals").set_text(total_string)
    
 
     def refresh_menu(self):
         #first populate the autocomplete - contains all entries in lowercase
         self.activities.clear()
         all_activities = storage.get_autocomplete_activities()
-
         for activity in all_activities:
             activity_category = activity['name']
             if activity['category']:
@@ -487,6 +552,7 @@ class HamsterApplet(object):
                                     activity['category'],
                                     activity_category])
 
+
         #now populate the menu - contains only categorized entries
         store = self.activity_list.get_model()
         store.clear()
@@ -503,7 +569,6 @@ class HamsterApplet(object):
                                  activity['category'],
                                  activity_category])
 
-
         # finally add TODO tasks from evolution to both lists
         tasks = hamster.eds.get_eds_tasks()
         for activity in tasks:
@@ -545,9 +610,6 @@ class HamsterApplet(object):
         self.load_day() # reload day each time before showing to avoid outdated last activity
         self.update_label() #update also label, otherwise we can get 1 minute difference in durations (due to timers refreshed once a minute)
 
-        if self.last_activity: #reset value of current_activity input, maybe we have some leftovers there
-            self.glade.get_widget('current_activity').set_text(self.last_activity['name'])        
-
         label_geom = self.button.get_allocation()
         window_geom = self.window.get_allocation()
         
@@ -565,11 +627,18 @@ class HamsterApplet(object):
 
         self.window.move(x, y)
 
+
         if self.last_activity and self.last_activity["end_time"] == None:
-            self.activity_list.child.set_text(self.last_activity["name"])
+            label = self.last_activity['name']
+            if self.last_activity['category'] != _("Unsorted"):
+                label += "@%s" %  self.last_activity['category']
+            self.activity_list.child.set_text(label)
+
             self.activity_list.child.select_region(0, -1)
+            self._gui.get_object("more_info_label").hide()
         else:
             self.activity_list.child.set_text('')
+            self._gui.get_object("more_info_label").show()
 
         # doing unstick / stick here, because sometimes while switching
         # between workplaces window still manages to dissappear
@@ -594,6 +663,12 @@ class HamsterApplet(object):
     def on_toggle(self, widget):
         dispatcher.dispatch('panel_visible', self.button.get_active())
 
+    def on_activity_list_key_pressed(self, entry, event):
+        #treating tab as keydown to be able to cycle through available values
+        if event.keyval == gtk.keysyms.Tab:
+            event.keyval = gtk.keysyms.Down
+        return False
+        
     def on_activity_switched(self, component):
         # do stuff only if user has selected something
         # for other cases activity_edited will be triggered
@@ -607,52 +682,25 @@ class HamsterApplet(object):
         
         if activity_name == "":
             return
-        self.add_fact(activity_name)
+        
+        storage.add_fact(activity_name)
+        dispatcher.dispatch('panel_visible', False)
 
     """listview events"""
-    def on_todays_keys(self, tree, event):
-        if (event.keyval == gtk.keysyms.Delete):
+    def on_todays_keys(self, tree, event_key):
+        if (event_key.keyval == gtk.keysyms.Delete):
             self.delete_selected()
             return True
-        elif (event.keyval == gtk.keysyms.e  \
-              and event.state & gtk.gdk.CONTROL_MASK):
-            self._open_edit_activity()
-            return True
-            
         return False
     
-    def _open_edit_activity(self):
-        """opens activity editor for selected row"""
-        selection = self.treeview.get_selection()
-        (model, iter) = selection.get_selected()
-        fact_id = model[iter][0]
-            
-        custom_fact = CustomFactController(None, fact_id)
-        custom_fact.show()        
-    
     def on_today_row_activated(self, tree, path, column):
-        if column == self.edit_column:
-            self._open_edit_activity()
-        else:
-            selection = tree.get_selection()
-            (model, iter) = selection.get_selected()
-            activity_name = model[iter][1].decode('utf8', 'replace')
-            if activity_name:
-                description = model[iter][5]
-                if description:
-                    description = description.decode('utf8', 'replace')
-                    activity_name = "%s, %s" % (activity_name, description)
-                    
-                self.add_fact(activity_name)
-    
-    def add_fact(self, activity_name):
-        if self.last_activity and self.last_activity['start_time'] != datetime.date.today():
-            # very seldom thing but we have been caught right after midnight
-            self.refresh_hamster()
-        storage.add_fact(activity_name)
-        dispatcher.dispatch('panel_visible', False)        
+        selection = tree.get_selection()
+        (model, iter) = selection.get_selected()
+        activity_name = model[iter][1].decode('utf8', 'replace')
+        if activity_name:
+            storage.add_fact(activity_name)
+            dispatcher.dispatch('panel_visible', False)        
         
-
         
     def on_windows_keys(self, tree, event_key):
         if (event_key.keyval == gtk.keysyms.Escape
@@ -671,14 +719,14 @@ class HamsterApplet(object):
 
     def on_overview(self, menu_item):
         dispatcher.dispatch('panel_visible', False)
-        stats_viewer = StatsViewer()
+        stats_viewer = StatsViewer(self)
         stats_viewer.show()
 
     def show_overview(self, menu_item, verb):
         return self.on_overview(menu_item)
 
     def on_custom_fact(self, menu_item):
-        custom_fact = CustomFactController()
+        custom_fact = CustomFactController(self)
         custom_fact.show()
 
     def on_about (self, component, verb):
@@ -693,7 +741,7 @@ class HamsterApplet(object):
         if self.preferences_editor and self.preferences_editor.window:
             self.preferences_editor.window.present()
         else:
-            self.preferences_editor = PreferencesEditor()
+            self.preferences_editor = PreferencesEditor(self)
             self.preferences_editor.show()
     
     """signals"""
@@ -703,9 +751,8 @@ class HamsterApplet(object):
         self.update_label()
     
     def after_fact_update(self, event, date):
-        if date.date() == datetime.date.today():
-            self.load_day()
-            self.update_label()
+        self.load_day()
+        self.update_label()
     
         self.__update_fact()
 
