@@ -24,7 +24,7 @@ pygtk.require('2.0')
 import os
 import gtk
 
-from hamster import dispatcher, storage, SHARED_DATA_DIR
+from hamster import dispatcher, storage, SHARED_DATA_DIR, stuff
 from hamster.Configuration import GconfStore
 
 def get_prev(selection, model):
@@ -89,8 +89,9 @@ class PreferencesEditor:
         ]
     
     
-    def __init__(self):
-        self.glade = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, "preferences.glade"))
+    def __init__(self, parent = None):
+        self.parent = parent
+        self._gui = stuff.load_ui_file("preferences.ui")
         self.config = GconfStore.get_instance()
         self.window = self.get_widget('preferences_window')
 
@@ -139,7 +140,7 @@ class PreferencesEditor:
 
         self.load_config()
 
-        self.glade.signal_autoconnect(self)
+        self._gui.connect_signals(self)
 
         # Allow enable drag and drop of rows including row move
         self.activity_tree.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
@@ -195,6 +196,14 @@ class PreferencesEditor:
                 self.activity_tree.set_cursor((i, ))
             i += 1
                 
+    def select_category(self, id):
+        model = self.category_tree.get_model()
+        i = 0
+        for row in model:
+            if row[0] == id:
+                self.category_tree.set_cursor((i, ))
+            i += 1
+                
     def on_activity_list_drag_motion(self, treeview, drag_context, x, y, eventtime):
         self.prev_selected_activity = None
         try:
@@ -270,12 +279,10 @@ class PreferencesEditor:
         
         if drop_info:
             path, position = drop_info
-
             iter = model.get_iter(path)
+            changed = storage.change_category(int(data), model[iter][0])
             
-            storage.change_category(int(data), model[iter][0])
-            
-            context.finish(True, True, etime)
+            context.finish(changed, True, etime)
         else:
             context.finish(False, True, etime)
 
@@ -286,7 +293,7 @@ class PreferencesEditor:
 
     def get_widget(self, name):
         """ skip one variable (huh) """
-        return self.glade.get_widget(name)
+        return self._gui.get_object(name)
 
     def get_store(self):
         """returns store, so we can add some watchers in case if anything changes"""
@@ -295,50 +302,46 @@ class PreferencesEditor:
     def show(self):
         self.window.show_all()
 
-    def update_activity(self, tree_row):
-        print "Update activity: ", tree_row[3]
-        #map to dictionary, for easier updates
-        name = tree_row[1]
-        category_id = tree_row[2]
-        id = tree_row[0]
-        
-        if id == -1:
-            #search for activities with same name
-            activities = storage.get_activities(category_id)
-            for activity in activities:
-                if activity['name'].lower() == name:
-                    self.select_activity(activity['id'])
-                    return False
-            
-            tree_row[0] = storage.add_activity(name, category_id)
-        else:
-            storage.update_activity(id, name, category_id)
-            
-        return True
-
     # callbacks
     def category_edited_cb(self, cell, path, new_text, model):
-        model[path][1] = new_text
         id = model[path][0]
-        name = model[path][1]
-        
-        if id == -2:
-            id = storage.add_category(name)
+        if id == -1:
+            return False #ignoring unsorted category
+
+        #look for dupes
+        categories = storage.get_category_list()
+        for category in categories:
+            if category['name'].lower() == new_text.lower():
+                if id == -2: # that was a new category
+                    self.category_store.remove(model.get_iter(path))
+                self.select_category(category['id'])
+                return False
+
+        if id == -2: #new category
+            id = storage.add_category(new_text.decode("utf-8"))
             model[path][0] = id
-        elif id > -1:  #ignore unsorted category (id = -1)
-            storage.update_category(id, name)
+        else:
+            storage.update_category(id, new_text)
+
+        model[path][1] = new_text
 
 
     def activity_name_edited_cb(self, cell, path, new_text, model):
+        id = model[path][0]
+        category_id = model[path][2]
+        
+        #look for dupes
+        activities = storage.get_activities(category_id)
+        for activity in activities:
+            if activity['name'].lower() == new_text.lower():
+                if id == -1: # that was a new category
+                    self.activity_store.remove(model.get_iter(path))
+                self.select_activity(activity['id'])
+                return False
+        
+        model[path][0] = storage.add_activity(new_text.decode("utf-8"), category_id)
         model[path][1] = new_text
-        res = self.update_activity(model[path])
-        
-        # if update fails on new activity - it's because it is a dupe
-        # so we remove it
-        if not res and model[path][0] == -1: 
-           self.activity_store.remove(model.get_iter(path))
-        
-        return res
+        return True
         
 
     def category_changed_cb(self, selection, model):
@@ -348,12 +351,16 @@ class PreferencesEditor:
         if iter == None:
             self.activity_store.clear()
         else:
+            self.prev_selected_activity = None
+
             id = model[iter][0]
             self.activity_store.load(model[iter][0])
         
-        #do not allow to remove the unsorted category
-        self.get_widget('promote_activity').set_sensitive(False)
-        self.get_widget('demote_activity').set_sensitive(False)
+        #start with nothing
+        self.get_widget('activity_up').set_sensitive(False)
+        self.get_widget('activity_down').set_sensitive(False)
+        self.get_widget('activity_edit').set_sensitive(False)
+        self.get_widget('activity_remove').set_sensitive(False)
 
         return True
 
@@ -373,14 +380,18 @@ class PreferencesEditor:
         
         # treat any selected case
         unsorted_selected = self._get_selected_category() == -1
-        self.get_widget('promote_activity').set_sensitive(False)
-        self.get_widget('demote_activity').set_sensitive(False)
+        self.get_widget('activity_up').set_sensitive(False)
+        self.get_widget('activity_down').set_sensitive(False)
+
+        self.get_widget('activity_edit').set_sensitive(iter != None)
+        self.get_widget('activity_remove').set_sensitive(iter != None)
+        
         if iter != None and not unsorted_selected:
             first_item = model.get_path(iter) == (0,)
-            self.get_widget('promote_activity').set_sensitive(not first_item)
+            self.get_widget('activity_up').set_sensitive(not first_item)
 
             last_item = model.iter_next(iter) == None
-            self.get_widget('demote_activity').set_sensitive(not last_item)
+            self.get_widget('activity_down').set_sensitive(not last_item)
 
     def _del_selected_row(self, tree):
         selection = tree.get_selection()
@@ -444,14 +455,26 @@ class PreferencesEditor:
             self.prev_selected_category = path
         
 
+    def on_activity_remove_clicked(self, button):
+        self.remove_current_activity()
+
+    def on_activity_edit_clicked(self, button):
+        self.activityCell.set_property("editable", True)
+
+        selection = self.activity_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        path = model.get_path(iter)[0]
+        self.activity_tree.set_cursor(path, focus_column = self.activityColumn, start_editing = True)
+        
+
+
     """keyboard events"""
     def on_activity_list_key_pressed(self, tree, event_key):
         key = event_key.keyval
         selection = tree.get_selection()
         (model, iter) = selection.get_selected()
         if (event_key.keyval == gtk.keysyms.Delete):
-            storage.remove_activity(model[iter][0])
-            self._del_selected_row(tree)
+            self.remove_current_activity()
 
         elif key == gtk.keysyms.F2 :
             self.activityCell.set_property("editable", True)
@@ -460,7 +483,25 @@ class PreferencesEditor:
             #tree.grab_focus()
             #tree.set_cursor(path, start_editing = True)
 
+    def remove_current_activity(self):
+        selection = self.activity_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        storage.remove_activity(model[iter][0])
+        self._del_selected_row(self.activity_tree)
 
+
+    def on_category_remove_clicked(self, button):
+        self.remove_current_category()
+        
+    def on_category_edit_clicked(self, button):
+        self.categoryCell.set_property("editable", True)
+
+        selection = self.category_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        path = model.get_path(iter)[0]
+        self.category_tree.set_cursor(path, focus_column = self.categoryColumn, start_editing = True)
+        
+        
     def on_category_list_key_pressed(self, tree, event_key):
         key = event_key.keyval
         
@@ -471,10 +512,7 @@ class PreferencesEditor:
         (model, iter) = selection.get_selected()
 
         if  key == gtk.keysyms.Delete:
-            id = model[iter][0]
-            if id != -1:
-                storage.remove_category(id)
-                self._del_selected_row(tree)
+            self.remove_current_category()
         elif key == gtk.keysyms.F2:
             self.categoryCell.set_property("editable", True)
             path = model.get_path(iter)[0]
@@ -482,12 +520,34 @@ class PreferencesEditor:
             #tree.grab_focus()
             #tree.set_cursor(path, start_editing = True)
 
+    def remove_current_category(self):
+        selection = self.category_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        id = model[iter][0]
+        if id != -1:
+            storage.remove_category(id)
+            self._del_selected_row(self.category_tree)
+
     def on_preferences_window_key_press(self, widget, event):
+        # ctrl+w means close window
+        if (event.keyval == gtk.keysyms.w \
+            and event.state & gtk.gdk.CONTROL_MASK):
+            self.close_window()
+
+        # escape can mean several things
         if event.keyval == gtk.keysyms.Escape:
-            self.window.destroy()     
+            #check, maybe we are editing stuff
+            if self.activityCell.get_property("editable"):
+                self.activityCell.set_property("editable", False)
+                return
+            if self.categoryCell.get_property("editable"):
+                self.categoryCell.set_property("editable", False)
+                return
+
+            self.close_window()     
 
     """button events"""
-    def on_add_category_clicked(self, button):
+    def on_category_add_clicked(self, button):
         """ appends row, jumps to it and allows user to input name """
         
         new_category = self.category_store.insert_before(self.category_store.unsorted_category,
@@ -500,7 +560,7 @@ class PreferencesEditor:
                                          start_editing = True)
 
 
-    def on_add_activity_clicked(self, button):
+    def on_activity_add_clicked(self, button):
         """ appends row, jumps to it and allows user to input name """
         category_id = self._get_selected_category()
         
@@ -514,31 +574,43 @@ class PreferencesEditor:
                                          focus_cell = None,
                                          start_editing = True)
 
-    def on_remove_activity_clicked(self, button):
+    def on_activity_remove_clicked(self, button):
         removable_id = self._del_selected_row(self.activity_tree)
         storage.remove_activity(removable_id)
 
-    def on_promote_activity_clicked(self, button):
+    def on_activity_up_clicked(self, button):
         (model, iter) = self.selection.get_selected()
 
         #previous item
         prev_iter = get_prev(self.selection, model)
-        storage.swap_activities(model[iter][0], model[prev_iter][0])
+        storage.swap_activities(model[iter][0], model[iter][3],
+                                model[prev_iter][0], model[prev_iter][3])
         model.move_before(iter, prev_iter)
 
         self.activity_changed(self.selection, model)
 
-    def on_demote_activity_clicked(self, button):
+    def on_activity_down_clicked(self, button):
         (model, iter) = self.selection.get_selected()
 
         next_iter = model.iter_next(iter)
-        storage.swap_activities(model[iter][0], model[next_iter][0])
+        storage.swap_activities(model[iter][0], model[iter][3],
+                                model[next_iter][0], model[next_iter][3])
         self.activity_store.move_after(iter, next_iter)
 
         self.activity_changed(self.selection, model)
 
     def on_close_button_clicked(self, button):
-        self.window.destroy()
+        self.close_window()
+
+    def on_close(self, widget, event):
+        self.close_window()        
+    
+    def close_window(self):
+        if not self.parent:
+            gtk.main_quit()
+        else:
+            self.window.destroy()
+            return False
 
     def on_shutdown_track_toggled(self, checkbox):
         self.config.set_stop_on_shutdown(checkbox.get_active())
@@ -549,10 +621,10 @@ class PreferencesEditor:
     def on_notify_interval_format_value(self, slider, value):
         if value <=120:
             # notify interval slider value label
-            label = _("%(interval_minutes)d minutes" % {'interval_minutes': value})
+            label = _(u"%(interval_minutes)d minutes") % {'interval_minutes': value}
         else:
             # notify interval slider value label
-            label = _("Never")
+            label = _(u"Never")
         
         return label
     
